;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Carrot Top Lifts (sprites B7 & B8), by imamelia
;;
;; This is a disassembly of sprites B7 and B8 in SMW, the Carrot Top Lifts.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, this sprite will act like sprite B7 and move to the
;; upper right.  If the first extra bit is set, it will act like sprite B8 and move to the
;; upper left.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!ExtraBit = $04	; the extra bit; normally 04, but may be set to 01 if using GEMS

Speed:
db $00,$F8,$00,$08

TileXDisp:
db $10,$00,$10,$00,$10,$00

TileYDisp:
db $00,$10,$10,$00,$10,$10

Tilemap:
db $E4,$E0,$E2,$E4,$E0,$E2

TileProp:
db $0B,$0B,$0B,$4B,$4B,$4B

Data038BAA:	; big old data table that I don't really have a good name for

db $20,$20,$20,$20,$20,$20,$20,$20
db $20,$20,$20,$20,$20,$20,$20,$20
db $20,$1F,$1E,$1D,$1C,$1B,$1A,$19
db $18,$17,$16,$15,$14,$13,$12,$11
db $10,$0F,$0E,$0D,$0C,$0B,$0A,$09
db $08,$07,$06,$05,$04,$03,$02,$01
db $00,$00,$00,$00,$00,$00,$00,$00
db $00,$00,$00,$00,$00,$00,$00,$00
db $00,$00,$00,$00,$00,$00,$00,$00
db $00,$00,$00,$00,$00,$00,$00,$00
db $01,$02,$03,$04,$05,$06,$07,$08
db $09,$0A,$0B,$0C,$0D,$0E,$0F,$10
db $11,$12,$13,$14,$15,$16,$17,$18
db $19,$1A,$1B,$1C,$1D,$1E,$1F,$20
db $20,$20,$20,$20,$20,$20,$20,$20
db $20,$20,$20,$20,$20,$20,$20,$20

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
LDA $7FAB10,x
AND #!ExtraBit
STA $1504,x	; make a backup of the extra bit (for LDY and whatnot)
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR CarrotTopLiftMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Return0:
RTS

CarrotTopLiftMain:

JSR CarrotTopLiftGFX

LDA $9D		;
BNE Return0	; return if sprites are locked

JSR SubOffscreenX0	;

LDA $1540,x	; if the sprite state timer has not run out...
BNE NoResetTimer	; don't reset it or change the sprite state
INC $C2,x	; increment the sprite state
LDA #$80		; 80 - fixed timer for each sprite state
STA $1540,x	;
NoResetTimer:	;
LDA $C2,x	;
AND #$03	; 4 possible indexes
TAY		;
LDA Speed,y	;
STA $B6,x		; set the sprite X speed
LDY $1504,x	; if the extra bit is clear...
BNE NoInvert	;
EOR #$FF		; then the sprite's Y speed is the inverse of its X speed
INC		;
NoInvert:		;
STA $AA,x	; if the extra bit is set, the sprite's X and Y speed are equal

JSL $01801A	; update sprite Y position without gravity

LDA $E4,x	; back up the sprite's X position?
STA $151C,x	; (not sure what the purpose of this is...)

JSL $018022	; update sprite X position without gravity

JSR SetUpClipping	; determine whether the sprite should be solid or not
JSL $03B69F	; get sprite clipping A routine
JSL $03B72B	; check for contact between the sprite and player
BCC Return1	; if there is none, just return

LDA $7D		;
BMI Return1	; return if the player is moving upward

LDA $94		;
SEC		;
SBC $151C,x	;
CLC		;
ADC #$1C	;
LDY $1504,x	;
BEQ NoAdd	;
CLC		;
ADC #$38	;
NoAdd:		;
TAY		;
LDA $187A	;
CMP #$01		;
LDA #$20		;
BCC NotOnYoshi	;
LDA #$30		; Well, I don't really have much to say here...
NotOnYoshi:	;
CLC		;
ADC $96		;
STA $00		;
LDA $D8,x	;
CLC		;
ADC Data038BAA,y	;
CMP $00		;
BPL Return1	; This entire portion of code simply handles the interaction.
LDA $187A	;
CMP #$01		;
LDA #$1D	;
BCC NotOnYoshi2	;
LDA #$2D	;
NotOnYoshi2:	;
STA $00		;
LDA $D8,x	;
CLC		;
ADC Data038BAA,y	; It's interesting, though...I doubt many SMW sprites have completely custom interaction.
PHP		;
SEC		;
SBC $00		;
STA $96		;
LDA $14D4,x	;
SBC #$00		;
PLP		;
ADC #$00	;
STA $97		; Offsetting XY positions and stuff...
STZ $7D		;
LDA #$01		;
STA $1471	;
LDY #$00		;
LDA $1491	;
BPL ZeroY		;
DEY		;
ZeroY:		;
CLC		; It could certainly be worse, though.  It's still less convoluted than the orange platforms' floating code.
ADC $94		;
STA $94		;
TYA		;
ADC $95		;
STA $95		;

Return1:
RTS

SetUpClipping:

; For want of a better name, this routine helps the sprite determine whether
; the player is interacting or not.

LDA $94		;
CLC		;
ADC #$04	;
STA $00		;
LDA $95		;
ADC #$00	; Once again, I really don't have any good comments.
STA $08		;
LDA #$08		;
STA $02		;
STA $03		;
LDA #$20		;
LDY $187A	; This code just takes the place of $03B6E5 (get sprite clipping B) or $03B664 (get player clipping).
BEQ NotOnYoshi3	;
LDA #$30		;
NotOnYoshi3:	;
CLC		;
ADC $96		;
STA $01		; It sets up some variables in scratch RAM for use in the contact-checking subroutine at $03B72B.
LDA $97		;
ADC #$00	;
STA $09		;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CarrotTopLiftGFX:

JSR GetDrawInfo

PHX		;
LDA $1504,x	; check if the extra bit is set
CMP #$01		;
LDX #$02		;
STX $02		; either way, we're drawing 3 tiles
BCC StartGFXLoop	; if the extra bit is set...
LDX #$05		; start the tile index at 05 instead of 02

StartGFXLoop:	;

LDA $00		;
CLC		;
ADC TileXDisp,x	; set the tile X displacement
STA $0300,y	;

LDA $01		;
CLC		;
ADC TileYDisp,x	; set the tile Y displacement
STA $0301,y	;

LDA Tilemap,x	;
STA $0302,y	; set the tile number

LDA TileProp,x	;
ORA $64		; set the tile properties
STA $0303,y	;

INY #4		;
DEX		;
DEC $02		; if we have more tiles to draw...
BPL StartGFXLoop	; loop the routine

PLX		;
LDY #$02		; the tiles are 16x16
LDA #$02		; and 3 tiles were drawn
JSL $81B7B3	;

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS











